@@ -9,6 +9,7 @@
 #include <QAudioFormat>
 #include <QAudioDevice>
 #include <QAudioBuffer>
+#include <QElapsedTimer>
 #include <QOperatingSystemVersion>
 #include <QRegularExpression>
 #include <qttexttospeech-config.h>
@@ -53,6 +54,7 @@ private slots:
 
     void sayMultiple_data();
     void sayMultiple();
+    void sayInvalid();
 
     void pauseAtUtterance_data();
     void pauseAtUtterance();
@@ -661,6 +663,33 @@ void tst_QTextToSpeech::sayWithRates()
     logger.dismiss();
 }
 
+void tst_QTextToSpeech::sayInvalid()
+{
+    QFETCH_GLOBAL(QString, engine);
+    if (engine != "mock" && !hasDefaultAudioOutput())
+        QSKIP("No audio device present");
+
+    QTextToSpeech tts(engine);
+    QTRY_COMPARE(tts.state(), QTextToSpeech::Ready);
+    selectWorkingVoice(&tts);
+    auto logger = qScopeGuard([&tts]{
+        qWarning() << "Failure with voice" << tts.voice();
+    });
+
+    int speakingCount = 0;
+    bool doneSpeaking = false;
+    connect(&tts, &QTextToSpeech::stateChanged, this, [&](QTextToSpeech::State state){
+        if (state == QTextToSpeech::Speaking)
+            ++speakingCount;
+        else if (state == QTextToSpeech::Ready)
+            doneSpeaking = true;
+    });
+
+    tts.say("©");
+    QTRY_VERIFY(doneSpeaking);
+    QTRY_COMPARE(speakingCount, 1);
+}
+
 void tst_QTextToSpeech::sayMultiple_data()
 {
     QTest::addColumn<QStringList>("textList");
@@ -792,14 +821,15 @@ void tst_QTextToSpeech::sayingWord()
 
     QElapsedTimer timer;
     QStringList words;
-    QList<qint64> times;
+
+    QList<std::chrono::nanoseconds> times;
     connect(&tts, &QTextToSpeech::sayingWord, this,
         [&words, &times, &timer, text](const QString &word, qsizetype id, qsizetype start, qsizetype length) {
         const QString &slice = text.sliced(start, length);
         QCOMPARE(word, slice);
         QCOMPARE(id, 0);
         words << text.sliced(start, length);
-        times << timer.elapsed();
+        times << timer.durationElapsed();
     });
 
     timer.start();
@@ -810,7 +840,7 @@ void tst_QTextToSpeech::sayingWord()
     });
     QTRY_COMPARE(tts.state(), QTextToSpeech::Speaking);
     QTRY_COMPARE(tts.state(), QTextToSpeech::Ready);
-    qint64 totalTime = timer.elapsed();
+    auto totalTime = timer.durationElapsed();
 
     QCOMPARE(words, expectedWords);
 
@@ -821,6 +851,13 @@ void tst_QTextToSpeech::sayingWord()
     if (words.count() > 1)
         QCOMPARE_GE(times.last(), totalTime * 0.4);
 
+    std::list<std::chrono::nanoseconds> timeDifferences;
+    std::adjacent_difference(times.begin(), times.end(), std::back_inserter(timeDifferences));
+    timeDifferences.pop_front(); // first element is always 0
+
+    for (auto timeDiff : timeDifferences)
+        QCOMPARE_GT(timeDiff, std::chrono::milliseconds(5));
+
     debugHelper.dismiss();
 }
 
@@ -840,6 +877,10 @@ void tst_QTextToSpeech::sayingWordWithPause()
     if (engine != "mock" && !hasDefaultAudioOutput())
         QSKIP("No audio device present");
 
+    if (engine == "flite")
+        QSKIP("QTBUG-137947 QTextToSpeech::pause(QTextToSpeech::BoundaryHint::Word) not implemented"
+              " for flite");
+
     QFETCH(QStringList, words);
     QFETCH(int, pauseAt);
 
